/* 2 September 2001, Alexandros Droseltis */

/* This file includes filters that produce a better image
   for the rows.

   Essential are:
   filter1

   Not essential are:
   ambitus, filter2, filter3.


   FUNCTION

   filter1
   It replaces all triple accidentals (double sharp+sharp, 
   double flat+flat) and their corresponding notes, 
   with the corresponding enharmonic notes.

   filter2
   It replaces all double accidentals and their corresponding
   notes, with the corresponding enharmonic notes.

   filter3
   It replaces all e & b sharps and c & f flats 
   with the corresponding enharmonic notes.

   ambitus
   It transposes the rows so that the smallest number of
   ledger lines is used at the output (c1<note<b2)

   Filter
   It is a package that apllies all the above filters
*/



void ambitus(int input[][12][2], int plithos);
void filter1(int input[][12][2], int plithos);
void filter2(int input[][12][2], int plithos);
void filter3(int input[][12][2], int plithos);
void Filter(int input[][12][2], int plithos, int opt[]);

void ambitus(int input[][12][2], int plithos)
{
  int i, j, temp[12][12][2], note, r, d;

  for(i=0; i<12; i++)
    {
      for(j=0; j<plithos; j++)
	{
	  note=input[i][j][1];
	  if(note<14)
	    {
	      d=-1;
	      break;
	    }
	  else if(note>27)
	    {
	      d=1;
	      break;
	    }
	  else d=0;
	}

      for(j=0; j<plithos; j++)
	{
	  temp[i][j][0]=input[i][j][0];
	}

      if(d==0)
	{
	  for(j=0; j<plithos; j++) temp[i][j][1]=input[i][j][1];
	}
      else if(d==-1)
	{
	  for(j=0; j<plithos; j++) temp[i][j][1]=input[i][j][1]+7;
	}
      else if(d==1)
	{
	  for(j=0; j<plithos; j++) temp[i][j][1]=input[i][j][1]-7;
	}
    }
  for(i=0; i<12; i++)
    {
      for(j=0; j<plithos; j++)
	{
	  input[i][j][0]=temp[i][j][0];
	  input[i][j][1]=temp[i][j][1];
	}
    }
}





void filter1(int input[][12][2], int plithos)
{
  int i, j, temp[12][12][2], acc, note;

  for(i=0; i<12; i++)
    {
      for(j=0; j<plithos; j++)
	{
	  acc=input[i][j][0];
	  note=input[i][j][1];
	  if(abs(acc)!=3)
	    {
	      temp[i][j][0]=acc;
	      temp[i][j][1]=note;
	    }
	  else
	    {
	      if(acc==3)
		{
		  if((note+7)%7!=0 && (note+7)%7!=3 && (note+7)%7!=4)
		    {
		      temp[i][j][0]=0;
		      temp[i][j][1]=note+2;
		    }
		  else
		    {

		      temp[i][j][0]=1;
		      temp[i][j][1]=note+1;
		    }
		}
	      else if(acc==-3)
		{
		  if((note+7)%7!=2 && (note+7)%7!=5 && (note+7)%7!=6)
		    {
		      temp[i][j][0]=0;
		      temp[i][j][1]=note-2;
		    }
		  else
		    {
		      temp[i][j][0]=-1;
		      temp[i][j][1]=note-1;
		    }
		}
	    }
	}
    }

  for(i=0; i<12; i++)
    {
      for(j=0; j<plithos; j++)
	{
	  input[i][j][0]=temp[i][j][0];
	  input[i][j][1]=temp[i][j][1];
	}
    }
}

void filter2(int input[][12][2], int plithos)
{
  int i, j, temp[12][12][2], acc, note;

  for(i=0; i<12; i++)
    {
      for(j=0; j<plithos; j++)
	{
	  acc=input[i][j][0];
	  note=input[i][j][1];
	  if(abs(acc)!=2)
	    {
	      temp[i][j][0]=acc;
	      temp[i][j][1]=note;
	    }
	  else
	    {
	      if(acc==2)
		{
		  temp[i][j][1]=note+1;
		  if((note+7)%7!=2 && (note+7)%7!=6)
		    {
		      temp[i][j][0]=0;
		      
		    }
		  else
		    {
		      temp[i][j][0]=1;
		    }
		}
	      else if(acc==-2)
		{
		  temp[i][j][1]=note-1;
		  if((note+7)%7!=0 && (note+7)%7!=3) temp[i][j][0]=0;
		  else temp[i][j][0]=-1;
		}
	    }
	}
    }

  for(i=0; i<12; i++)
    {
      for(j=0; j<plithos; j++)
	{
	  input[i][j][0]=temp[i][j][0];
	  input[i][j][1]=temp[i][j][1];
	}
    }
}

void filter3(int input[][12][2], int plithos)
{
  int i, j, temp[12][12][2], acc, note;

  for(i=0; i<12; i++)
    {
      for(j=0; j<plithos; j++)
	{
	  acc=input[i][j][0];
	  note=input[i][j][1];
	  /* If the accidental is a sharp */
	  if(acc==1)
	    {
	      /* If the note is neither E nor B */
	      if((note+7)%7!=2 && (note+7)%7!=6)
		{
		  temp[i][j][0]=acc;
		  temp[i][j][1]=note;
		}
	      /* If the note is E or B */
	      else
		{
		  temp[i][j][0]=0;
		  temp[i][j][1]=note+1;
		}
	    }
	  /* If the accidental is a flat */
	  else if(acc==-1)
	    {
	      /* If the note is neither C nor F */
	      if((note+7)%7!=0 && (note+7)%7!=3)
		{
		  temp[i][j][0]=acc;
		  temp[i][j][1]=note;
		}
	      /* If the note is C or F */
	      else
		{
		  temp[i][j][0]=0;
		  temp[i][j][1]=note-1;
		}
	    }
	  /* If the accidental is a natural */
	  else
	    {
	      temp[i][j][0]=acc;
	      temp[i][j][1]=note;
	    }
	}
    }


  for(i=0; i<12; i++)
    {
      for(j=0; j<plithos; j++)
	{
	  input[i][j][0]=temp[i][j][0];
	  input[i][j][1]=temp[i][j][1];
	}
    }
}

void Filter(int input[][12][2], int plithos, int opt[])
{
  ambitus(input, plithos);
  ambitus(input, plithos);
  filter1(input, plithos);
  if(opt[3]) filter2(input, plithos);
  if(opt[2]) filter3(input, plithos);
}

